from collections.abc import Sequence
from typing import Any

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.db.models import F
from django.template.loader import render_to_string
from django_tasks import task

from terminusgps_notifications.models import (
    MessagePackage,
    TerminusgpsNotificationsCustomer,
)


@task
def send_email(
    to: Sequence[str],
    subject: str,
    template_name: str,
    reply_to: Sequence[str] = ("support@terminusgps.com",),
    from_email: str = "noreply@terminusgps.com",
    bcc: Sequence[str] | None = None,
    cc: Sequence[str] | None = None,
    context: dict[str, Any] | None = None,
    html_template_name: str | None = None,
) -> bool:
    """
    Sends an email to target addresses.

    :param to: Sequence of destination email addresses.
    :type to: ~collections.abc.Sequence[str]
    :param subject: Subject line.
    :type subject: str
    :param template_name: Email template name.
    :type template_name: str
    :param reply_to: Reply-to email address. Default is ``("support@terminusgps.com",)``.
    :type reply_to: str
    :param from_email: Origin email address. Default is :py:obj:`None`
    :type from_email: str | None
    :param cc: Sequence of closed-copy email addresses.
    :type cc: ~collections.abc.Sequence[str] | None
    :param bcc: Sequence of blind closed-copy email addresses.
    :type bcc: ~collections.abc.Sequence[str] | None
    :param context: Email context. Default is :py:obj:`None`.
    :type context: dict[str, ~typing.Any] | None
    :param html_template_name: Email HTML template name. If provided, attaches HTML alternative to the email. Default is :py:obj:`None`.
    :type html_template_name: str | None

    """
    msg = EmailMultiAlternatives(
        subject=subject,
        body=render_to_string(template_name, context=context),
        from_email=from_email,
        to=to,
        cc=cc if cc else [],
        bcc=bcc if bcc else [admin[1] for admin in settings.ADMINS],
        reply_to=reply_to,
    )
    if html_template_name is not None:
        html_content = render_to_string(html_template_name, context=context)
        msg.attach_alternative(html_content, "text/html")
    return bool(msg.send(fail_silently=True))


@task
def reset_customer_messages(customer_pk: int) -> None:
    """
    Sets the customer's :py:attr:`messages_count` to 0.

    :param customer_pk: Customer primary key.
    :type customer_pk: int
    :returns: Nothing.
    :rtype: None

    """
    try:
        customer = TerminusgpsNotificationsCustomer.objects.get(pk=customer_pk)
        customer.messages_count = 0
        customer.save(update_fields=["messages_count"])
    except TerminusgpsNotificationsCustomer.DoesNotExist:
        return


@task
def reset_customer_packages(customer_pk: int) -> None:
    """
    Deletes fully used customer packages.

    :param customer_pk: Customer primary key.
    :type customer_pk: int
    :returns: Nothing.
    :rtype: None

    """
    packages = MessagePackage.objects.filter(customer__pk=customer_pk)
    packages.filter(count__gte=F("max")).delete()



import typing
import urllib.parse

from django.contrib.auth.models import AbstractBaseUser
from django.contrib.auth.views import LoginView as LoginViewBase
from django.contrib.auth.views import LogoutView as LogoutViewBase
from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_control, cache_page
from django.views.generic import FormView, RedirectView, TemplateView
from terminusgps.mixins import HtmxTemplateResponseMixin

from terminusgps_notifications import tasks
from terminusgps_notifications.forms import (
    TerminusgpsNotificationsAuthenticationForm,
    TerminusgpsNotificationsRegistrationForm,
)
from terminusgps_notifications.models import TerminusgpsNotificationsCustomer


class SourceCodeView(RedirectView):
    http_method_names = ["get"]
    permanent = True
    url = "https://github.com/terminusgps/terminusgps-notifications/"


class DocumentationView(RedirectView):
    http_method_names = ["get"]
    permanent = True
    url = "https://terminusgps.github.io/terminusgps-notifications/"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class NavbarView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    partial_template_name = "terminusgps_notifications/partials/_navbar.html"
    template_name = "terminusgps_notifications/navbar.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class HomeView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    extra_context = {
        "title": "Terminus GPS Notifications",
        "subtitle": "We know where ours are... do you?",
    }
    http_method_names = ["get"]
    partial_template_name = "terminusgps_notifications/partials/_home.html"
    template_name = "terminusgps_notifications/home.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class TermsView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    extra_context = {
        "title": "Terms & Conditions",
        "subtitle": "You agree to these by using Terminus GPS services",
    }
    template_name = "terminusgps_notifications/terms.html"
    partial_template_name = "terminusgps_notifications/partials/_terms.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class ContactView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    extra_context = {"title": "Contact", "subtitle": "Drop us a line"}
    template_name = "terminusgps_notifications/contact.html"
    partial_template_name = "terminusgps_notifications/partials/_contact.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class PrivacyView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    extra_context = {
        "title": "Privacy Policy",
        "subtitle": "How we use your data",
    }
    http_method_names = ["get"]
    template_name = "terminusgps_notifications/privacy.html"
    partial_template_name = "terminusgps_notifications/partials/_privacy.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class LoginView(HtmxTemplateResponseMixin, LoginViewBase):
    content_type = "text/html"
    extra_context = {
        "title": "Login",
        "subtitle": "We know where ours are... do you?",
    }
    form_class = TerminusgpsNotificationsAuthenticationForm
    http_method_names = ["get", "post"]
    partial_template_name = "terminusgps_notifications/partials/_login.html"
    success_url = reverse_lazy("terminusgps_notifications:dashboard")
    template_name = "terminusgps_notifications/login.html"

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        if username := self.request.GET.get("username"):
            initial["username"] = username
        return initial


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class LogoutView(HtmxTemplateResponseMixin, LogoutViewBase):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    template_name = "terminusgps_notifications/logged_out.html"
    partial_template_name = (
        "terminusgps_notifications/partials/_logged_out.html"
    )


class RegisterView(HtmxTemplateResponseMixin, FormView):
    content_type = "text/html"
    extra_context = {
        "title": "Register",
        "subtitle": "You'll know where yours are...",
    }
    form_class = TerminusgpsNotificationsRegistrationForm
    http_method_names = ["get", "post"]
    partial_template_name = "terminusgps_notifications/partials/_register.html"
    template_name = "terminusgps_notifications/register.html"

    @transaction.atomic
    def form_valid(
        self, form: TerminusgpsNotificationsRegistrationForm
    ) -> HttpResponse:
        user = form.save(commit=True)
        user.first_name = form.cleaned_data["first_name"]
        user.last_name = form.cleaned_data["last_name"]
        user.email = form.cleaned_data["username"]
        user.save(update_fields=["first_name", "last_name", "email"])
        customer = TerminusgpsNotificationsCustomer(user=user)
        customer.company = form.cleaned_data["company_name"]
        customer.save()
        self.send_registration_complete_email(user)
        return HttpResponseRedirect(
            reverse(
                "terminusgps_notifications:login",
                query={"username": form.cleaned_data["username"]},
            )
        )

    @staticmethod
    def send_registration_complete_email(user: AbstractBaseUser) -> None:
        subject = "Terminus GPS Notifications - Registration Complete"
        template = "terminusgps_notifications/emails/registration_complete.txt"
        context = {
            "first_name": user.first_name,
            "link_account": urllib.parse.urljoin(
                "https://api.terminusgps.com/",
                reverse("terminusgps_notifications:account"),
            ),
            "link_dashboard": urllib.parse.urljoin(
                "https://api.terminusgps.com/",
                reverse("terminusgps_notifications:dashboard"),
            ),
            "link_homepage": urllib.parse.urljoin(
                "https://api.terminusgps.com/",
                reverse("terminusgps_notifications:home"),
            ),
            "link_notifications": urllib.parse.urljoin(
                "https://api.terminusgps.com/",
                reverse("terminusgps_notifications:notifications"),
            ),
            "link_subscription": urllib.parse.urljoin(
                "https://api.terminusgps.com/",
                reverse("terminusgps_notifications:subscription"),
            ),
        }
        tasks.send_email.enqueue(
            to=[user.email],
            subject=subject,
            template_name=template,
            context=context,
        )



# Generated by Django 5.2.9 on 2025-12-02 21:42

import django.core.validators
import terminusgps.wialon.flags
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifications', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wialonnotification',
            name='name',
            field=models.CharField(help_text='Provide a memorable name.', max_length=64, validators=[django.core.validators.MinLengthValidator(4)]),
        ),
        migrations.AlterField(
            model_name='wialontoken',
            name='flags',
            field=models.PositiveIntegerField(default=terminusgps.wialon.flags.TokenFlag['ONLINE_TRACKING'] | terminusgps.wialon.flags.TokenFlag['VIEW_ACCESS'] | terminusgps.wialon.flags.TokenFlag['MANAGE_NONSENSITIVE'] | terminusgps.wialon.flags.TokenFlag['MANAGE_SENSITIVE']),
        ),
    ]



import urllib.parse

from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser
from django.urls import reverse
from terminusgps_payments.models import CustomerProfile

from terminusgps_notifications.models import (
    TerminusgpsNotificationsCustomer,
    WialonToken,
)


def get_customer(
    user: AbstractBaseUser,
) -> TerminusgpsNotificationsCustomer | None:
    """
    Returns the :py:obj:`~terminusgps_notifications.models.TerminusgpsNotificationsCustomer` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A customer object, if the user has one.
    :rtype: ~terminusgps_notifications.models.TerminusgpsNotificationsCustomer | None

    """
    try:
        if user.is_authenticated:
            return TerminusgpsNotificationsCustomer.objects.get(user=user)
    except TerminusgpsNotificationsCustomer.DoesNotExist:
        return


def get_customer_profile(user: AbstractBaseUser) -> CustomerProfile | None:
    """
    Returns the :py:obj:`~terminusgps_payments.models.CustomerProfile` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A customer profile object, if the user has one.
    :rtype: ~terminusgps_payments.models.CustomerProfile | None

    """
    try:
        if user.is_authenticated:
            return CustomerProfile.objects.get(user=user)
    except CustomerProfile.DoesNotExist:
        return


def get_wialon_token(user: AbstractBaseUser) -> str | None:
    """
    Returns the :py:obj:`~terminusgps_notifications.models.WialonToken` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A Wialon API token, if the user has one.
    :rtype: str | None

    """
    try:
        if user.is_authenticated:
            token = WialonToken.objects.get(customer__user=user)
            return getattr(token, "name", None)
    except WialonToken.DoesNotExist:
        return


def get_wialon_login_parameters(username: str) -> str:
    """
    Returns a query string of path parameters to be added to the Wialon authentication request.

    :param username: A username to autofill into the authentication form.
    :type username: str
    :returns: A query string.
    :rtype: str

    """
    return urllib.parse.urlencode(
        {
            "client_id": "Terminus GPS Notifications",
            "access_type": settings.WIALON_TOKEN_ACCESS_TYPE,
            "activation_time": 0,
            "duration": 0,
            "lang": "en",
            "flags": 0x1,
            "user": username,
            "response_type": "token",
            "redirect_uri": urllib.parse.urljoin(
                "https://api.terminusgps.com/"
                if not settings.DEBUG
                else "http://127.0.0.1:8000/",
                reverse("terminusgps_notifications:account"),
            ),
        }
    )



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import os
import sys

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "src.settings.prod")
sys.path.insert(0, os.path.abspath("../../"))
django.setup()

project = "terminusgps-notifications"
copyright = "2025, Terminus GPS, LLC"
author = "Terminus GPS, LLC"
release = "1.2.0"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "autoclasstoc",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "django": (
        "https://docs.djangoproject.com/en/5.2/",
        "https://docs.djangoproject.com/en/5.2/objects.inv",
    ),
}

templates_path = ["_templates"]
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

autodoc_member_order = "groupwise"
html_theme = "sphinxawesome_theme"
pygments_style = "sas"
pygments_style_dark = "lightbulb"
html_static_path = ["_static"]



# Generated by Django 6.0 on 2025-12-16 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifications', '0002_alter_wialonnotification_name_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='wialonnotification',
            name='resource_name',
            field=models.CharField(blank=True, max_length=50),
        ),
    ]
